#!/bin/sh

#  SystemAudioScript.sh
#  SystemAudioCapture
#
#  Created by vitacy tan on 9/3/14.
#

install_to_dir="/System/Library/Extensions"

daemon_plist_name="com.greatdy.driver.startup.plist"
daemon_path="/Library/Application Support/Greatdy/SystemAudio"

daemon_script_name="SystemAudioScript.sh"
launch_daemon_path="/Library/LaunchDaemons"
kext_src_name="ComputerAudioCapture.kext"
kext_install_name="$kext_src_name"

install_path="$daemon_path/$kext_install_name"
install_path="$install_to_dir/$kext_install_name"

src_dir=$(dirname "$0")

function is_os_yosemite()
{
    version="$(cat /System/Library/CoreServices/SystemVersion.plist|plutil -p -|grep ProductVersion|cut -d. -f 2)"
    test "$version" = "10"
}
function uninstall_launch_daemon()
{
    rm "$launch_daemon_path/$daemon_plist_name"
    rm "$daemon_path/*"
}
function install_launch_daemon()
{

    uninstall_launch_daemon
    mkdir -p "$daemon_path"

    cp "$src_dir/$daemon_script_name" "$daemon_path"
    cp "$src_dir/$daemon_plist_name" "$daemon_path"
    ln -s "$daemon_path/$daemon_plist_name" "$launch_daemon_path/$daemon_plist_name"
}
function install_kext_files()
{
    mkdir -p "$daemon_path"
    cp -R "$src_dir/$kext_src_name"  "$install_path"
}
function uninstall_kext_files()
{
    rm -rf "$install_to_dir/$kext_install_name"
    rm -rf "$daemon_path/$kext_install_name"
}
function boot_on_kext_dev_mode()
{
    boot_args="$(nvram boot-args 2>/dev/null |cut -f 2-)"
    if ! [[ "$boot_args" == *"kext-dev-mode="* ]]
    then
        nvram boot-args="kext-dev-mode=1 $boot_args"
    fi
    nvram boot-args
}
function uninstall()
{
    kextunload "$install_path"
    uninstall_kext_files
    uninstall_launch_daemon
    echo "uninstall ok $?"
}
function install()
{
    uninstall
    install_kext_files
    kextload "$install_path"
    kext_fail_code="$?"
    echo "install ok $kext_fail_code"
    if test "$kext_fail_code" = 71
    then
        boot_on_kext_dev_mode
        echo "kext need reboot 1"
    fi

}

function verify()
{
    codesign --verify
}



function is_loaded()
{
    local kext_id="com.Greatdy.driver.SystemAudioCapture"
    kextstat -lb "$kext_id"| grep -q "$kext_id"
}
function start()
{
    if [ -d "$install_path" ] && ! is_loaded
    then
        sudo kextload "$install_path"
    fi
}


function run_service()
{
    case "$1" in
            "install")
                install "$2"
                exit $?;
                ;;
            "uninstall")
                uninstall
                exit $?;
                ;;
            "start")
                start
                exit $?;
                ;;
            "status")
                is_loaded
                exit $?;
                ;;
            **)
                echo "Error: Unknown action '$1'"
                exit 1;
    esac
}

run_service "$1"
